/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.stage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.pojo.SaBaseLoginUser;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.listener.CommonDataChangeEventCenter;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.core.enums.BaseDataTypeEnum;
import vip.xiaonuo.modular.stage.param.PigpenStageAddParam;
import vip.xiaonuo.modular.stage.param.PigpenStageEditParam;
import vip.xiaonuo.modular.stage.param.PigpenStageIdParam;
import vip.xiaonuo.modular.stage.param.PigpenStagePageParam;
import vip.xiaonuo.modular.stage.service.PigpenStageService;
import vip.xiaonuo.modular.stage.entity.PigpenStage;
import vip.xiaonuo.modular.stage.mapper.PigpenStageMapper;


import java.util.List;

/**
 * 栋舍阶段Service接口实现类
 *
 * @author newspaper
 * @date  2023/12/05 15:12
 **/
@Service
public class PigpenStageServiceImpl extends ServiceImpl<PigpenStageMapper, PigpenStage> implements PigpenStageService {

    @Override
    public Page<PigpenStage> page(PigpenStagePageParam pigpenStagePageParam) {
        QueryWrapper<PigpenStage> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(pigpenStagePageParam.getSortField(), pigpenStagePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigpenStagePageParam.getSortOrder());
            queryWrapper.orderBy(true, pigpenStagePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigpenStagePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigpenStage::getSortCode);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PigpenStageAddParam pigpenStageAddParam) {
        SaBaseLoginUser user = StpLoginUserUtil.getLoginUser();
        PigpenStage pigpenStage = BeanUtil.toBean(pigpenStageAddParam, PigpenStage.class);
        pigpenStage.setOrgId(user.getOrgId());
        boolean repeatName = this.count(new LambdaQueryWrapper<PigpenStage>().eq(PigpenStage::getStageName, pigpenStage.getStageName()).eq(PigpenStage::getOrgId, user.getOrgId())) > 0;
        if (repeatName){
            throw new CommonException("存在重复的栋舍阶段，名称为：{}", pigpenStage.getStageName());
        }
        this.save(pigpenStage);

        //发布增加事件
        CommonDataChangeEventCenter.doAddWithData(BaseDataTypeEnum.STAGE.getValue(), JSONUtil.createArray().put(pigpenStage));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PigpenStageEditParam pigpenStageEditParam) {
        SaBaseLoginUser user = StpLoginUserUtil.getLoginUser();
        PigpenStage pigpenStage = this.queryEntity(pigpenStageEditParam.getId());
        BeanUtil.copyProperties(pigpenStageEditParam, pigpenStage);
        boolean repeatName = this.count(new LambdaQueryWrapper<PigpenStage>().eq(PigpenStage::getStageName, pigpenStage.getStageName()).eq(PigpenStage::getOrgId, user.getOrgId())) > 0;
        if (repeatName){
            throw new CommonException("存在重复的栋舍阶段，名称为：{}", pigpenStage.getStageName());
        }
        pigpenStage.setUpdateBy(user.getId());
        this.updateById(pigpenStage);

        //发布更新事件
        CommonDataChangeEventCenter.doUpdateWithData(BaseDataTypeEnum.STAGE.getValue(), JSONUtil.createArray().put(pigpenStage));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigpenStageIdParam> pigpenStageIdParamList) {
        List<String> toDeleteStageIdList = CollStreamUtil.toList(pigpenStageIdParamList, PigpenStageIdParam::getId);
        // 执行删除
        this.removeByIds(toDeleteStageIdList);

        //发布删除事件
        CommonDataChangeEventCenter.doDeleteWithDataId(BaseDataTypeEnum.STAGE.getValue(),toDeleteStageIdList);
    }

    @Override
    public PigpenStage detail(PigpenStageIdParam pigpenStageIdParam) {
        return this.queryEntity(pigpenStageIdParam.getId());
    }

    @Override
    public PigpenStage queryEntity(String id) {
        PigpenStage pigpenStage = this.getById(id);
        if(ObjectUtil.isEmpty(pigpenStage)) {
            throw new CommonException("栋舍阶段不存在，id值为：{}", id);
        }
        return pigpenStage;
    }

    @Override
    public List<PigpenStage> getAllPigpenList() {
        SaBaseLoginUser user = StpLoginUserUtil.getLoginUser();
        List<PigpenStage> pigpenStages = this.list(new LambdaQueryWrapper<PigpenStage>().eq(PigpenStage::getOrgId, user.getOrgId()).orderByAsc(PigpenStage::getSortCode));
        return pigpenStages;
    }
}
